<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoAuxilioMoradia extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nreventom             = "100106712";
        $dsobservacao          = $itens['DSOBSERVACAOAUXIMORADIA']   ?? null;
        $vreventofixo          = $itens['VREVENTOFIXO']   ?? null;
        $idtipomovimento       = "1";
        $dtinivigencia    = isset($itens['DTINIVIGENCIA']) ? DateUtil::getDataDeString(substr($itens['DTINIVIGENCIA'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtfimvigencia    = isset($itens['DTFIMVIGENCIA']) ? DateUtil::getDataDeString(substr($itens['DTFIMVIGENCIA'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtatual               = DateUtil::getDataDeString(substr(date('d/m/Y'),0,10), DateUtil::FORMATO_BRASILEIRO, true);
        $nrorg                 = Utils::getParamters()->getNrorg();
        $nrvinculom            = $this->getSolicitacao()->getNrvinculom();

        $evento = $this->verificaExistEventoAtivo($nrorg, $nrvinculom, $nreventom, $dtfimvigencia, $dtatual);

        if($evento) {
            throw new \Exception("Já há um auxílio moradia registrado para esse vínculo.");
        }

        $fpaVincevento = Repositories::FPA_VINCEVENTO;
        $fpaVincevento = new $fpaVincevento;
        $fpaVincevento->setNrorg($nrorg);
        $fpaVincevento->setNrvinculom($nrvinculom);
        $fpaVincevento->setDtinivigencia($dtinivigencia);
        $fpaVincevento->setDtfimvigencia($dtfimvigencia);
        $fpaVincevento->setNreventom($nreventom);
        $fpaVincevento->setIdtipomovimento($idtipomovimento);
        $fpaVincevento->setVreventofixo($vreventofixo);
        $fpaVincevento->setDsobservacao($dsobservacao);
        $fpaVincevento->setDtinclusao(DateUtil::getDataAtual(false));
        $fpaVincevento->setDtultatu($dtatual);
        $fpaVincevento->setNrorginclusao(Utils::getParamters()->getNrorg());
        $fpaVincevento->setNrorgultatu(Utils::getParamters()->getNrorg());
        $fpaVincevento->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $fpaVincevento->setCdoperultatu(Utils::getParamters()->getCdoperador());
        $fpaVincevento->setNrvincevento(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'FPA_VINCEVENTO', 1, 12, true));
        Utils::getEntityManager()->persist($fpaVincevento);
        Utils::getEntityManager()->flush();
    }
    
    public function rejeitar() {}

    public function verificaExistEventoAtivo($nrorg, $nrvinculom, $nreventom, $dtfimvigencia, $dtatual) {
        $gpeAltesitufunc = Repositories::FPA_VINCEVENTO;
        $sql = <<<SQL
            SELECT a
            FROM $gpeAltesitufunc a
            WHERE a.nrorg = :NRORG
            AND a.nrvinculom = :NRVINCULOM
            AND a.nreventom = :NREVENTOM
            AND (
                a.dtfimvigencia >= :DTATUAL OR
                a.dtfimvigencia IS NULL
            )
  
SQL;
        $params = array(
            'NRORG'          => $nrorg,
            'NRVINCULOM'     => $nrvinculom,
            'NREVENTOM'      => $nreventom,
            'DTATUAL'        => $dtatual
        );

        $result = Utils::getEntityManager()->createQuery($sql)->execute($params);

        return empty($result) ? $result : $result[0];
    }
}

