<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

use Teknisa\Model\Gestaopessoas\Entities\GpeTipotransfer;
use Teknisa\Model\Geral\Entities\Tipoestrutura;

class ProcessamentoSolicitacaoRemanejamento extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrorg          = Utils::getParamters()->getNrorg();
        $nrvinculom     = $this->getSolicitacao()->getNrvinculom();
        $dtcompetencia  = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(),DateUtil::FORMATO_BRASILEIRO,true);
        
        $paramOrg = Utils::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->findOneBy(['nrorg' => $nrorg]);
        $nrtipoestrutlegal = !empty($paramOrg) ? $paramOrg->getNrtpestrutlegal() : null;

        // Transferência/Remanejamento
        $nrtipotransfer = $itens['NRTIPOTRANSFER']  ?? null;
        $nrtpmovtransfm = $itens['NRTPMOVTRANSFM']  ?? null;
        $nrmotivotransf = $itens['NRMOTIVOTRANSF']  ?? null;
        $nrestruturam   = $itens['NRESTRUTURAM']    ?? null;
        $dtinimoviment  = isset($itens['DTINIMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTINIMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtfimmoviment  = isset($itens['DTFIMMOVIMENT']) ? DateUtil::getDataDeString(substr($itens['DTFIMMOVIMENT'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
    
        $estrutura_nova = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $nrestruturam));
        $result = Utils::getEntityManager()->getRepository(Repositories::GPE_MOVIMENTACAO)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtinimoviment" => $dtinimoviment));

        // SALÁRIO
        $dtaltesalario  = isset($itens['DTALTESALARIO']) ? DateUtil::getDataDeString(substr($itens['DTALTESALARIO'],0,10), 'd/m/Y', true) : null;
        $nrtipoaltera   = $itens['NRTIPOALTERA']        ?? null;
        $idtpsalario    = $itens['IDTPSALARIO']         ?? null;
        $nrocorrencia   = $itens['NROCORRENCIA']        ?? null;
        $nrtpmodalidsal = $itens['NRTPMODALIDSAL']  ?? null; 
        $novoVlrSalario = $itens['VRSALARIO']           ?? $itens['NOVOVALORSALARIO'] ?? null;

        // ESCALA
        $nrturno        = $itens['NRTURNO']             ??  null;
        $nrmotivoescala = $itens['NRMOTIVOESCALA']      ??  null;
        $nrescalatrabm  = $itens['NRESCALATRABM']       ??  null;
        $dtiniescala    = isset($itens['DTINIESCALA']) ? DateUtil::getDataDeString(substr($itens['DTINIESCALA'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtcompetenciaString  = Utils::getParamters()->getDtmescomptencDiaCorrente();


        $gpevinculoH        = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrvinculom, $dtcompetencia->format('d/m/Y'));
        $nrestruturalegal   = $gpevinculoH->getNrestrutlegal();
        $nrestruturageren   = $gpevinculoH->getNrestrutgeren();

        // Validações Estruruta
        if($nrestruturam == $nrestruturalegal || $nrestruturam == $nrestruturageren) {
            throw new \Exception("Este vínculo já está registrado nesta Estrutura.");
        }

        // Valida se o tipo de transferência é para uma estrutura de um tipo compatível.
        $this->verificaTipoTransferencia($nrtipotransfer, $estrutura_nova->getNrtipoestrutura());

        if($result){
            $estrutura_historico = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array("nrorg"=> $nrorg,"nrestruturam" => $result->getNrestruturam()));

            if($estrutura_historico->getNrtipoestrutura() == $estrutura_nova->getNrtipoestrutura()) {
                throw new \Exception("Já existe uma movimentação para este tipo de estrutura nesta data");
            }
        }
        if($nrtipotransfer == 1){
            if($estrutura_nova->getNrtipoestrutura() != $nrtipoestrutlegal && $estrutura_nova->getNrtipoestrutura() != 10) {
                throw new \Exception("A nova estrutura legal deve ser do mesmo tipo que a antiga.");
            }
        }
        else if($estrutura_nova->getNrtipoestrutura() == $nrtipoestrutlegal) {
            throw new \Exception("A nova estrutura legal deve ser de tipo diferente que a antiga.");
        }
   
        if($dtcompetenciaString !== $itens['DTINIMOVIMENT'] && !DateUtil::comparaDatas($dtinimoviment, DateUtil::getDataDeString($dtcompetenciaString), DateUtil::DEPOIS)){
                throw new \Exception("A movimentação não pode ser retroativa");
        }

        // Validações Escala
        if($nrescalatrabm && $nrturno && $dtiniescala) {
            $result = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTEESCALA)->findBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom,"dtiniescala" => $dtiniescala));

            if($result) {
                throw new \Exception("Já existe uma alteração de escala para este vinculo nesta data");
            }
        }

        // Validações Salario
        if($nrtipoaltera && $novoVlrSalario && $idtpsalario) {
            if (!$dtaltesalario) {
                throw new \Exception("Solicitação sem data ou com data de alteração inválida. Favor reprovar e incluir uma nova solicitação.");
            }
            
            if (!$nrtipoaltera) {
                throw new \Exception("Solicitação sem tipo de alteração informado. Favor reprovar e incluir uma nova solicitação.");
            }
        }


        // Cria a Movimentação
        ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_nova, $nrestruturam, $nrmotivotransf, $nrtipotransfer, $nrtpmovtransfm, $dtfimmoviment, $dtinimoviment);


        if($nrtipotransfer == 1) {
            $dtfimmovimentsaida = DateUtil::subtraiIntervalo($dtinimoviment, 1)->format('d/m/Y');
            $dtfimmovimentsaida = DateUtil::getDataDeString(substr($dtfimmovimentsaida,0,10), DateUtil::FORMATO_BRASILEIRO, true);

            ProcessamentoSolicitacaoUtil::novaMovimentacao($nrorg, $nrvinculom, $estrutura_nova, $nrestruturam, $nrmotivotransf, $nrtipotransfer, "9", $dtfimmovimentsaida, $dtinimoviment);
        }
        
        $vinc = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrvinculom, $dtcompetencia);
        if($vinc->getDtmescompetenc() == $dtcompetencia){
            $vinc->setDtultatu(DateUtil::getDataAtual());
            $vinc->setNrorgultatu(Utils::getParamters()->getNrorg());
            $vinc->setCdoperultatu(Utils::getParamters()->getCdoperador());
            if($nrtipotransfer == 1) {
                $vinc->setNrestrutlegal($nrestruturam);
            }   
            else {
                $vinc->setNrestrutgeren($nrestruturam);
            }
            Utils::getEntityManager()->persist($vinc);
            Utils::getEntityManager()->flush();
        }
        else {
            ProcessamentoSolicitacaoUtil::novoHistoricoVinculo($nrorg, $nrvinculom, $dtcompetencia, $vinc->getNrsitufuncm(), $vinc->getNrcargo(), $vinc->getNrescalatrabm(),
                $vinc->getIdcontribuisind(), $vinc->getNrdependir(), $vinc->getNrdependsfam(), $vinc->getDtbaseferias(), $vinc->getIdremuneracao(),
                $vinc->getIdtppagamento(), (($nrtipotransfer == 1)?$nrestruturam:$vinc->getNrestrutlegal()),(($nrtipotransfer == 1)?$vinc->getNrestrutgeren():$nrestruturam) , $vinc->getNrtpmodalidsal(), $vinc->getIdmultvinc(),
                $vinc->getNrfuncao(), $vinc->getCdcontribindividual(), $vinc->getNrtpmovtransfm(), $vinc->getDtfimcontrdetermin(),
                $vinc->getNrestrutsind(), $vinc->getConfidencial()
            );
        }


        if($nrescalatrabm && $nrturno && $dtiniescala) {            
            ProcessamentoSolicitacaoUtil::atualizaAlteEscala(
                $nrorg,
                $nrvinculom,
                $nrescalatrabm,
                DateUtil::subtraiIntervalo($dtiniescala,1,DateUtil::DIA)->format(DateUtil::FORMATO_BRASILEIRO),
                DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),
                Utils::getParamters()->getCdoperador()
            );

            ProcessamentoSolicitacaoUtil::atualizaEscalaVinculoh(
                $nrorg,
                $nrvinculom,
                $nrescalatrabm,
                DateUtil::getDataAtual(true)->format(DateUtil::FORMATO_BRASILEIRO),
                Utils::getParamters()->getCdoperador()
            );
            
            $alteEscala = ProcessamentoSolicitacaoUtil::novoAlteEscala($nrorg, $nrvinculom, $nrturno, $nrmotivoescala, $nrescalatrabm, $dtiniescala);
        }


        if($nrtipoaltera && $novoVlrSalario && $idtpsalario) {
            $alteSalario = ProcessamentoSolicitacaoUtil::novoAlteSalario($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia, $nrtipoaltera, $nrtpmodalidsal, $idtpsalario, $novoVlrSalario);
        }
        
    }

    public function verificaTipoTransferencia($nrtipotransfer, $nrtipoestrutura){
        try {
            $paramOrg = Utils::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg' => Utils::getParamters()->getNrorg()));
            if ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFLEGAL) {
                if ($nrtipoestrutura != $paramOrg->getNrtpestrutlegal() && $nrtipoestrutura != Tipoestrutura::SINDICATO) {
                    throw new \Exception('Movimentaçao legal incompatível com o tipo de estrutura', 30);
                }
            } elseif ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFGEREN) {
                if (($nrtipoestrutura == $paramOrg->getNrtpestrutlegal() && $paramOrg->getNrtpestrutlegal() != $paramOrg->getNrtpestrutgeren()) || $nrtipoestrutura == Tipoestrutura::SINDICATO) {
                    throw new \Exception('Movimentaçao gerencial incompatível com o tipo de estrutura', 30);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function rejeitar() {}
}