<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class ValeTransporteRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function setValeTransporte(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO VAB_LINHAVINC (
                NRLINHAVINC,        NRLINHATARIFA,      NRVINCULOM,
                NRTIPOLINHAVINC,    NRSENTIDOLINHA,     DTINILINHAVINC,
                DTFIMLINHAVINC,     QTLINHATARIFA,      DSOBSLINHAVINC,
                IDATIVO,            NRORGINCLUSAO,      DTINCLUSAO,
                CDOPERINCLUSAO,     NRORGULTATU,        DTULTATU,
                CDOPERULTATU,       NRORG,              NRINTEGTRANSPM
            )
            VALUES (
                :NRLINHAVINC,       :NRLINHATARIFA,     :NRVINCULOM,
                :NRTIPOLINHAVINC,   :NRSENTIDOLINHA,    :DTINILINHAVINC,
                :DTFIMLINHAVINC,    :QTLINHATARIFA,     :DSOBSLINHAVINC,
                :IDATIVO,           :NRORGINCLUSAO,     :DTINCLUSAO,
                :CDOPERINCLUSAO,    :NRORGULTATU,       :DTULTATU,
                :CDOPERULTATU,      :NRORG,             :NRINTEGTRANSPM
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getLinhasVinculoVT(
        int $nrOrg,
        int $nrVinculom,
        string $dtAtual
    ): array {
        $sql = <<< SQL
            SELECT
                VBLV.*,
                VBT.VRTARIFA
            FROM
                VAB_LINHAVINC VBLV
                JOIN VAB_TARIFA VBT ON VBT.NRTARIFA = VBLV.NRLINHATARIFA
            WHERE
                VBLV.NRORG = :P_NRORG AND
                VBLV.NRVINCULOM = :P_NRVINCULOM AND
                (VBLV.DTFIMLINHAVINC >= :P_DTATUAL OR VBLV.DTFIMLINHAVINC IS NULL)
SQL;

        $params = [
            'P_NRORG'      => $nrOrg,
            'P_NRVINCULOM' => $nrVinculom,
            'P_DTATUAL'    => $dtAtual
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getTarifasVT(
        string $dtAtual
    ): array {
        $sql = <<< SQL
            SELECT
                *
            FROM
                VAB_TARIFA VBT
            WHERE
                (VBT.DTFIMTARIFA >= :P_DTATUAL OR VBT.DTFIMTARIFA IS NULL)
SQL;

        $params = [
            'P_DTATUAL'    => $dtAtual
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
}